diff scripts/general/cart2sph.m @ 8533:fb1b87ea4af9

Permit scalars when transforming coordinates.
author Ben Abbott <bpabbott@mac.com>
date Sat, 17 Jan 2009 17:32:11 -0500
parents aaff46fef256
children eb63fbe60fab
line wrap: on
line diff
--- a/scripts/general/cart2sph.m	Sat Jan 17 11:41:25 2009 -0500
+++ b/scripts/general/cart2sph.m	Sat Jan 17 17:32:11 2009 -0500
@@ -19,7 +19,7 @@
 ## -*- texinfo -*-
 ## @deftypefn {Function File} {[@var{theta}, @var{phi}, @var{r}] =} cart2sph (@var{x}, @var{y}, @var{z})
 ## Transform cartesian to spherical coordinates.
 ## @var{x}, @var{y} and @var{z} must be of same shape.
 ## @var{theta} describes the angle relative to the x-axis.
 ## @var{phi} is the angle relative to the xy-plane.
 ## @var{r} is the distance to the origin (0, 0, 0).
@@ -29,20 +29,60 @@
 ## Author: Kai Habel <kai.habel@gmx.de>
 ## Adapted-by: jwe
 
-function [Theta, Phi, R] = cart2sph (X, Y, Z)
+function [theta, phi, r] = cart2sph (x, y, z)
 
   if (nargin != 3)
     print_usage ();
   endif
 
-  if (! (ismatrix (X) && ismatrix (Y) && ismatrix (Z))
-      || ! size_equal (X, Y)
-      || ! size_equal (X, Z))
-    error ("cart2sph: arguments must be matrices of same size");
+  if ((ismatrix (x) && ismatrix (y) && ismatrix (z))
+      && (size_equal (x, y) || isscalar (x) || isscalar (y))
+      && (size_equal (x, z) || isscalar (x) || isscalar (z))
+      && (size_equal (y, z) || isscalar (y) || isscalar (z)))
+
+    theta = atan2 (y, x);
+    phi = atan2 (z, sqrt (x .^ 2 + y .^ 2));
+    r = sqrt (x .^ 2 + y .^ 2 + z .^ 2);
+
+  else
+    error ("cart2sph: arguments must be matrices of same size, or scalar");
   endif
 
-  Theta = atan2 (Y, X);
-  Phi = atan2 (Z, sqrt (X .^ 2 + Y .^ 2));
-  R = sqrt (X .^ 2 + Y .^ 2 + Z .^ 2);
+endfunction
+
+%!test
+%! x = [0, 1, 2];
+%! y = [0, 1, 2];
+%! z = [0, 1, 2];
+%! [t, p, r] = cart2sph (x, y, z);
+%! assert (t, [0, pi/4, pi/4], eps);
+%! assert (p, [0, 1, 1]*atan(sqrt(0.5)), eps);
+%! assert (r, [0, 1, 2]*sqrt(3), eps);
 
-endfunction
+%!test
+%! x = 0;
+%! y = [0, 1, 2];
+%! z = [0, 1, 2];
+%! [t, p, r] = cart2sph (x, y, z);
+%! assert (t, [0, 1, 1] * pi/2, eps);
+%! assert (p, [0, 1, 1] * pi/4, eps);
+%! assert (r, [0, 1, 2] * sqrt(2), eps);
+
+%!test
+%! x = [0, 1, 2];
+%! y = 0;
+%! z = [0, 1, 2];
+%! [t, p, r] = cart2sph (x, y, z);
+%! assert (t, [0, 0, 0]);
+%! assert (p, [0, 1, 1] * pi/4);
+%! assert (r, [0, 1, 2] * sqrt(2));
+
+%!test
+%! x = [0, 1, 2];
+%! y = [0, 1, 2];
+%! z = 0;
+%! [t, p, r] = cart2sph (x, y, z);
+%! assert (t, [0, 1, 1] * pi/4);
+%! assert (p, [0, 0, 0]);
+%! assert (r, [0, 1, 2] * sqrt(2));
+
@@ -19,7 +19,7 @@
 ## @deftypefn {Function File} {[@var{theta}, @var{phi}, @var{r}] =} cart2sph (@var{x}, @var{y}, @var{z})
 ## Transform cartesian to spherical coordinates.
-## @var{x}, @var{y} and @var{z} must be of same shape.
+## @var{x}, @var{y} and @var{z} must be of same shape, or scalar.
 ## @var{theta} describes the angle relative to the x-axis.
 ## @var{phi} is the angle relative to the xy-plane.